SystemOrganization addCategory: #UnificationTests!

TestCase subclass: #MatchingTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'UnificationTests'!

!MatchingTest methodsFor: 'helpers' stamp: 'fbs 7/22/2011 07:55'!
assert: left equals: right description: msg
	self
		assert: left = right
		description: ('{1} , expected {2}, got {3}' format: {msg. left printString. right printString.}).! !

!MatchingTest methodsFor: 'testing - matching' stamp: 'fbs 7/22/2011 08:41'!
testFirstMatchWins
	| result |
	result := (Leaf value: 1)
		matchOneOf:
			{[Leaf value: 1] -> ['first match'].
			[Leaf value: 1] -> ['second match'].}
		otherwise: ['otherwise block'].
	self assert: result = 'otherwise block' description: 'Otherwise block not invoked'.! !

!MatchingTest methodsFor: 'testing - matching' stamp: 'fbs 7/22/2011 13:19'!
testMatchPassesUnifierIntoBlock
	| result mgu |
	result := (Leaf value: 1)
		matchOneOf:
			{[Leaf value: #x asVariable] -> [:vars | mgu := vars. 'match']}
		otherwise: ['otherwise block'].
	self assert: result = 'match' description: 'No match found'.
	self assert: mgu notNil description: 'MGU not passed into block'.
	self assert: (mgu at: #x asVariable) = 1 description: 'Unexpected MGU'.! !

!MatchingTest methodsFor: 'testing - matching' stamp: 'fbs 7/22/2011 08:41'!
testMatchingTriesAllBlocks
	| result |
	result := (Leaf value: 1)
		matchOneOf:
			{[Tree empty] -> ['first match'].
			[Leaf value: 1] -> ['second match'].}
		otherwise: ['otherwise block'].
	self assert: result = 'otherwise block' description: 'Otherwise block not invoked'.! !

!MatchingTest methodsFor: 'testing - matching' stamp: 'fbs 7/22/2011 13:20'!
testMatchingUnifiesBidirectionally
	| result mgu |
	result := (Node left: #x asVariable right: (Leaf value: 1))
		matchOneOf:
			{[Node left: (Leaf value: 2) right: #y asVariable] -> [:vars | mgu := vars. 'match']}
		otherwise: ['otherwise block'].
	self assert: result = 'match' description: 'No match found'.
	self assert: mgu notNil description: 'MGU not passed into block'.
	self assert: (mgu at: #x asVariable) equals: (Leaf value: 2) description: 'Unexpected MGU (x)'.
	self assert: (mgu at: #y asVariable) equals: (Leaf value: 1) description: 'Unexpected MGU (y)'.! !

!MatchingTest methodsFor: 'testing - matching' stamp: 'fbs 7/22/2011 08:41'!
testNoMatchEvaluatesOtherwiseBlock
	| result |
	result := (Leaf value: 1)
		matchOneOf:
			{[Tree empty] -> ['matched something']}
		otherwise: ['otherwise block'].
	self assert: result = 'otherwise block' description: 'Otherwise block not invoked'.! !

!MatchingTest methodsFor: 'testing - matching' stamp: 'fbs 7/22/2011 08:42'!
testNoMatchEvaluatesRaisesError
	self should: [(Leaf value: 1)
		matchOneOf:
			{[Tree empty] -> ['matched something']}]
		raise: Error.! !

TestCase subclass: #MetaVariableTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'UnificationTests'!

!MetaVariableTest methodsFor: 'testing' stamp: 'fbs 5/24/2011 11:03'!
testUnapplyShouldDestructureCorrectly
	self assert: (MetaVariable named: #a) unapply = {#Variable. #a.}! !

!MetaVariableTest methodsFor: 'testing' stamp: 'fbs 5/24/2011 22:08'!
testVariableHasNoChildren
	self deny: (MetaVariable named: #foo) hasChildren.

	self assert: (MetaVariable named: #foo) children isEmpty.! !

!MetaVariableTest methodsFor: 'testing' stamp: 'fbs 5/25/2011 22:32'!
testVariableIsVariable
	self assert: (MetaVariable named: #foo) isMetaVariable.! !

TestCase subclass: #UnificationTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'UnificationTests'!

!UnificationTest methodsFor: 'testing' stamp: 'fbs 5/29/2011 15:09'!
testCyclicStructuresCantUnify
	| y |
	y := MetaVariable named: #y.
	self should: [y unifyWith: (Leaf value: y)] raise: UnificationFailure.! !

!UnificationTest methodsFor: 'testing' stamp: 'fbs 5/27/2011 20:28'!
testDifferentArityTermsShouldNotUnify
	"f(x) ~= g(x,y)"
	|f g |
	f := VariadicTerm with: #(#x).
	g := VariadicTerm with: #(#x #y).
	self should: [f unifyWith: g] raise: UnificationFailure.! !

!UnificationTest methodsFor: 'testing' stamp: 'fbs 5/29/2011 15:09'!
testDifferentConstantsShouldNotUnify
	self should: [(Leaf value: 1) unifyWith: (Leaf value: 2)] raise: UnificationFailure.! !

!UnificationTest methodsFor: 'testing' stamp: 'fbs 5/29/2011 15:06'!
testImpossibleUnificationShouldNotUnify
	self should: [(Leaf value: 2) unifyWith: (Leaf value: 1)] raise: UnificationFailure.
	self should: [1 unifyWith: 2] raise: UnificationFailure.! !

!UnificationTest methodsFor: 'testing' stamp: 'fbs 5/29/2011 15:08'!
testSelfUnificationShouldUnify
	| x |
	x := MetaVariable named: #x.
	"Tautologies"
	self assert: (1 unifyWith: 1) isEmpty.
	self assert: (x unifyWith: x) isEmpty.! !

!UnificationTest methodsFor: 'testing' stamp: 'fbs 5/26/2011 07:43'!
testTrivialBidirectionalUnificationShouldUnify
	| matches x y |
	x := MetaVariable named: #x.
	y := MetaVariable named: #y.
	matches := x unifyWith: y.
	self assert: matches = (Dictionary newFrom: {x -> y}).! !

!UnificationTest methodsFor: 'testing' stamp: 'fbs 5/29/2011 15:08'!
testTrivialUnificationsShouldUnify
	| matches x leaf |
	leaf := MetaVariable named: #leaf.
	matches := leaf unifyWith: (Leaf value: 1).
	self assert: matches = (Dictionary newFrom: {leaf -> (Leaf value: 1)}).
	
	x := MetaVariable named: #x.
	self assert: (x unifyWith: 2) = (Dictionary newFrom: {x -> 2}).
	self assert: (2 unifyWith: x) = (Dictionary newFrom: {x -> 2}).! !

!UnificationTest methodsFor: 'testing' stamp: 'fbs 5/29/2011 15:11'!
testTwoVariableBidirectionalUnificationShouldUnifyBoth
	"node(left, leaf(2)) =? node(leaf(1), right)"
	| matches left right |
	left := MetaVariable named: #left.
	right := MetaVariable named: #right.
	matches := (Node left: left right: (Leaf value: 2))
		unifyWith: (Node left: (Leaf value: 1) right: right).
	self assert: matches = (Dictionary newFrom: {
		left -> (Leaf value: 1).
		right -> (Leaf value: 2)}).! !

!UnificationTest methodsFor: 'testing' stamp: 'fbs 5/26/2011 07:51'!
testTwoVariableUnificationShouldUnifyBoth
	| matches left right |
	left := MetaVariable named: #left.
	right := MetaVariable named: #right.
	matches := (Node left: left right: right)
		unifyWith: (Node left: (Leaf value: 1) right: (Leaf value: 2)).
	self assert: matches = (Dictionary newFrom: {
		left -> (Leaf value: 1).
		right -> (Leaf value: 2)}).! !

!UnificationTest methodsFor: 'testing' stamp: 'fbs 5/26/2011 07:52'!
testUnificationWorksOnDeepStructures
	| matches x y |
	x := MetaVariable named: #x.
	y := MetaVariable named: #y.
	matches := (Node left: (Node left: (Node left: x))) unifyWith: (Node left: y).
	self assert: matches = (Dictionary newFrom: {y -> (Node left: (Node left: x))}).! !

!UnificationTest methodsFor: 'testing' stamp: 'fbs 5/26/2011 08:24'!
testVariableUnifyingAgainstTwoVariables
	| matches x y z |
	x := MetaVariable named: #x.
	y := MetaVariable named: #y.
	z := MetaVariable named: #z.
	matches := (Node left: x right: x) unifyWith: (Node left: y right: z).
	self assert: matches = (Dictionary newFrom: {y -> z. x -> z}).! !

AlgebraicDataType subclass: #VariadicTerm
	instanceVariableNames: 'subterms'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'UnificationTests'!

!VariadicTerm commentStamp: 'fbs 5/29/2011 16:47' prior: 0!
A general term that may contain any (variable) number of subterms. It's mainly just for testing.!

!VariadicTerm class methodsFor: 'instance variables' stamp: 'fbs 5/27/2011 20:26'!
with: aCollection
	^ self new with: aCollection.! !

!VariadicTerm methodsFor: 'destructuring' stamp: 'fbs 5/27/2011 20:27'!
unapply
	^ #Term asOrderedCollection, subterms.! !

!VariadicTerm methodsFor: 'private' stamp: 'fbs 5/27/2011 20:26'!
with: aCollection
	subterms := aCollection! !
